//***************************************************************************//
//* File Name: frustum.hpp                                                  *//
//* Author:    Tom Portegys, portegys@ilstu.edu                             *//
//* Date Made: 07/25/02                                                     *//
//* File Desc: Class declaration representing a camera frustum.             *//
//* Rev. Date:                                                              *//
//* Rev. Desc:                                                              *//
//*                                                                         *//
//***************************************************************************//

#ifndef __FRUSTUM_HPP__
#define __FRUSTUM_HPP__

#include "point3d.h"

class Frustum
{
  public:

	// Box/frustum bounding plane.
	struct Plane
	{
		float a,b,c,d;	// Parameters.
	};

	// Constructors.
	Frustum()
	{
		ExtractFrustum();
	}
	Frustum(struct Plane *planes)
	{
		register int i;

		for (i = 0; i < 6; i++)
		{
			this->planes[i] = planes[i];
		}
	}

	// Box/frustum bounding planes.
	struct Plane planes[6];

	// In frustum?
	bool intersects(float xmin, float xmax, float ymin, float ymax, float zmin, float zmax);
	bool isInside(Point3D vertex);

	// Plane to point distance.
	float planeToPoint(struct Plane, Point3D point);

  private:

	  // Extract frustum.
	  void ExtractFrustum();

};
#endif